/* normal.c - gaussian random numbers
 *
 * Copyright (C) 2001, 2005  Jochen Voss.  */

static const  char  rcsid[] = "$Id: normal.c 6506 2005-07-06 23:56:55Z voss $";

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <math.h>
#include <assert.h>

#include "jvrand-p.h"
#include "normal-tab.h"


double
jvrand_standard_normal (void)
/* This is a variation of the Ziggurat algorithm described by
 * George Marsaglia and Wai Wan Tsang.  */
{
  unsigned long  U, sigma, i, j;
  double  x, y;

  while (1) {
    U = jvrand_int32 ();
    i = U & 0x0000007F;
    sigma = U & 0x00000080;
    j = U>>8;

    x = j*wtab[i];
    if (j < ktab[i])  break;

    if (i<LEVELS-1) {
      double  y0, y1;
      y0 = ytab[i];
      y1 = ytab[i+1];
      y = y1+(y0-y1)*jvrand_real2();
    } else {
      x = PARAM_R - log(1.0-jvrand_real2())/PARAM_R;
      y = exp(-PARAM_R*(x-0.5*PARAM_R))*jvrand_real2();
    }
    if (y < exp(-0.5*x*x))  break;
  }
  return  sigma ? x : -x;
}

double
jvrand_normal (double m, double v)
/* Gaussian random numbers with mean M and variance V.  */
{
  assert (v >= 0);
  return  m + sqrt(v) * jvrand_standard_normal();
}
