/* timer.c - measure the speed of random number generation
 *
 * Copyright (C) 2001  Jochen Voss.  */

static const  char  rcsid[] = "$Id: timer.c 6506 2005-07-06 23:56:55Z voss $";

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <setjmp.h>
#include <sys/time.h>

#if JVRAND
#include "jvrand.h"
#else
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>

static gsl_rng *rnd;
static const gsl_rng_type *T;
#endif


static  sigjmp_buf  leave_loop;

static void
handle_alarm (int signum)
{
  siglongjmp (leave_loop, 1);
}

static unsigned long
count (void)
{
  volatile  unsigned long  n = 0;
  struct itimerval  pause;
  
  pause.it_interval.tv_usec = 0;
  pause.it_interval.tv_sec = 0;
  pause.it_value.tv_usec = 0;
  pause.it_value.tv_sec = 1;

  signal (SIGVTALRM, handle_alarm);
  if (sigsetjmp (leave_loop, 1) == 0) {
    if (setitimer (ITIMER_VIRTUAL, &pause, NULL) < 0)  abort ();
    while (1) {
#if JVRAND
      jvrand_standard_normal ();
#else
      gsl_ran_gaussian (rnd, 1);
#endif
      ++n;
    }
  }

  return  n;
}

int
main ()
{
  int  i;

#if JVRAND
  jvrand_init ();
  puts ("testing jvrand_standard_normal:");
#else
  gsl_rng_env_setup();
  T = gsl_rng_default;
  rnd = gsl_rng_alloc (T);
  puts ("testing gsl_ran_gaussian:");
#endif
  
  count ();
  for (i=0; i<10; ++i) {
    printf ("  %lu numbers/second\n", count ());
  } 
  return  0;
}
